#include "stdafx.h"
#include "HookDllInterface.h"
#include "OSInfo.h"

#if defined(UNICODE) || defined(_UNICODE)

#ifdef _DEBUG
#define HOOKDLL_ID	IDR_PWDSPYHKUD
#else	// _DEBUG
#define HOOKDLL_ID	IDR_PWDSPYHKUR
#endif	// _DEBUG

#else	// UNICODE

#ifdef _DEBUG
#define HOOKDLL_ID	IDR_PWDSPYHKD
#else	// _DEBUG
#define HOOKDLL_ID	IDR_PWDSPYHKR
#endif	// _DEBUG

#endif	// UNICODE

//***********************************************
CHookDllInterface::CHookDllInterface()
	: m_hModHookDll(NULL), m_szPwdSpyHkDll(NULL), m_pfnInstallHook(NULL), m_pfnRemoveHook(NULL), m_pfnScanPassword(NULL)
{
	COSInfo osi;

	if(osi.IsNT() && osi.GetMajor() >= 5)
	{
		TCHAR szTempPath[_MAX_PATH];

		m_szPwdSpyHkDll = new TCHAR[_MAX_PATH];
		GetTempPath(sizeof(szTempPath) / sizeof(TCHAR), szTempPath);
		GetTempFileName(szTempPath, _T("tmp"), 0, m_szPwdSpyHkDll);
		ExtractResource(AfxGetInstanceHandle(), HOOKDLL_ID, m_szPwdSpyHkDll);

		m_hModHookDll = LoadLibrary(m_szPwdSpyHkDll);
		if(m_hModHookDll)
		{
			m_pfnInstallHook = (PFNINSTALLHOOK)GetProcAddress(m_hModHookDll, "InstallHook"); _ASSERTE(m_pfnInstallHook);
			m_pfnRemoveHook = (PFNREMOVEHOOK)GetProcAddress(m_hModHookDll, "RemoveHook"); _ASSERTE(m_pfnRemoveHook);
			m_pfnScanPassword = (PFNSCANPASSWORD)GetProcAddress(m_hModHookDll, "ScanPassword"); _ASSERTE(m_pfnScanPassword);
		}
	}
}

//***********************************************
CHookDllInterface::~CHookDllInterface()
{
	// Make absolutely sure we unhook before closing
	if(m_pfnRemoveHook)
		m_pfnRemoveHook();

	if(m_hModHookDll)
		FreeLibrary(m_hModHookDll);

	if(m_szPwdSpyHkDll)
	{
		DeleteFile(m_szPwdSpyHkDll);
		delete[] m_szPwdSpyHkDll;
	}
}

//***********************************************
bool CHookDllInterface::ExtractResource(const HINSTANCE hInstance, const WORD nID, PCTSTR szFilename)
{
	_ASSERTE(hInstance);
	_ASSERTE(nID);
	_ASSERTE(szFilename);
	_ASSERTE(lstrlen(szFilename));

	bool bSuccess = false;

	try
	{
		HRSRC hResource = FindResource(hInstance, MAKEINTRESOURCE(nID), _T("BINARY")); _ASSERTE(hResource);
		HGLOBAL hFileResource = LoadResource(hInstance, hResource); _ASSERTE(hFileResource);
		PVOID pFile = LockResource(hFileResource);

		DWORD dwSize = SizeofResource(hInstance, hResource);

		// Open the file and filemap
		HANDLE hFile = CreateFile(szFilename, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		HANDLE hFilemap = CreateFileMapping(hFile, NULL, PAGE_READWRITE, 0, dwSize, NULL);

		// Get a pointer to write to
		PVOID pBaseAddress = MapViewOfFile(hFilemap, FILE_MAP_WRITE, 0, 0, 0);

		// Write the file
		CopyMemory(pBaseAddress, pFile, dwSize);

		// Unmap the file and close the handles
		UnmapViewOfFile(pBaseAddress);
		CloseHandle(hFilemap);
		CloseHandle(hFile);
	}
	catch(...) {}

	return bSuccess;
}

//***********************************************
bool CHookDllInterface::InstallHook(const DWORD dwThreadId)
{
	return (m_pfnInstallHook == NULL) ? false : m_pfnInstallHook(dwThreadId);
}

//***********************************************
bool CHookDllInterface::RemoveHook(void)
{
	return (m_pfnRemoveHook == NULL) ? false : m_pfnRemoveHook();
}

//***********************************************
bool CHookDllInterface::ScanPassword(const HWND hWnd, const HWND hPwdSpyWnd)
{
	return (m_pfnScanPassword == NULL) ? false : m_pfnScanPassword(hWnd, hPwdSpyWnd);
}
